package com.winhc.bigdata.spark.jobs.judicial

import com.winhc.bigdata.spark.udf.{BaseFunc, CaseReasonAggs, CompanyMapping, CourtRank, NameAggs}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.EsQueryUtils.updateAliases
import com.winhc.bigdata.spark.utils.{BaseUtil, EsQueryUtils, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.collection.mutable

/**
 * @Description:司法案件预处理
 * @author π
 * @date 2020/9/17 14:45
 */
object JudicialCaseRelationPreNew {
  def main(args: Array[String]): Unit = {
    var project = ""
    var ds = ""
    var c = ""
    if (args.length == 3) {
      val Array(p1, p2, p3) = args
      project = p1
      ds = p2
      c = p3
    } else {
      println("please check project ds c!")
      sys.exit(-1)
    }
    if (ds.equals("all")) ds = ""
    println(
      s"""
         |project: $project
         |ds: $ds
         |c: $c
         |""".stripMargin)

    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> s"$project",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10000"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    val r = JudicialCaseRelationPreNew(spark, project, ds)
    c match {
      case "calc" => r.calc()
      case "precalc" => r.precalc()
      case "updateIndex" => r.updateIndex()
      case "createIndex" => r.createIndex()
    }

    spark.stop()
  }
}


case class JudicialCaseRelationPreNew(s: SparkSession, project: String, ds: String
                                     ) extends LoggingUtils with CompanyMapping with BaseFunc with CourtRank {
  override protected val spark: SparkSession = s


  private def getStrToMap(cols: Seq[String]): String = {
    val set = cols.toSet
    val str = set.map(e => {
      s"concat_ws('\001','$e',cast($e as string))"
    }).mkString(",")
    s"str_to_map(concat_ws('\002',$str),'\002','\001')"
  }

  def precalc(): Unit = {
    case_no_trim_udf()
    prepareFunctions(spark)
    val t1 = s"$project.inc_ads_company_court_announcement"
    var t1_ds = ds
    if (StringUtils.isBlank(ds)) {
      t1_ds = BaseUtil.getPartion(t1, spark)
    }

    //法院公告
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_announcement')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |     select
         |     *,row_number() over(partition by detail_id order by date desc) num
         |     from (
         |           select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"2" as flag
         |            ,title(plaintiff,litigant,'法院公告') as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,null as case_reason
         |            ,case_no
         |            ,court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(litigant) as bg_name
         |            ,concat_ws(' ',publish_date,'00:00:00') as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_announcement
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"2" as flag
         |            ,title(plaintiff,litigant,'法院公告') as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,null as case_reason
         |            ,case_no
         |            ,court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(litigant) as bg_name
         |            ,concat_ws(' ',publish_date,'00:00:00') as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_announcement
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

    //文书预处理
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='wenshu')
         |SELECT
         |        COALESCE(a.judicase_id,b.new_judicase_id) judicase_id
         |        ,'0' as flag
         |        ,title
         |        ,case_type
         |        ,case_reason
         |        ,case_no
         |        ,court_name
         |        ,concat_ws('',case_stage(case_no)) as case_stage
         |        ,regexp_replace(yg_name,'\n',',') as yg_name
         |        ,regexp_replace(bg_name,'\n',',') as bg_name
         |        ,judge_date as date
         |        ,case_id as detail_id
         |        ,case_amt*10000
         |FROM    (
         |            SELECT  rowkey,max(judicase_id) judicase_id
         |            FROM $project.ads_judicial_case_relation_graph WHERE flag = 'wenshu_detail'
         |            GROUP BY rowkey
         |        ) a
         |RIGHT JOIN (
         |            SELECT *,md5(cleanup(case_no_trim(case_no))) as new_judicase_id
         |             FROM
         |            (
         |              SELECT  *,row_number() over(partition by docid order by judge_date desc) num
         |              FROM    winhc_eci.ods_wenshu_detail
         |              WHERE   ds > '0' AND case_no_trim(case_no) is not null
         |            )c
         |            where num = 1
         |        ) b
         |ON      a.rowkey = b.case_id
         |""".stripMargin).show(10, false)


    //开庭公告预处理
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_open_announcement')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |    select
         |    *,
         |    row_number() over(partition by detail_id order by date desc) num
         |    from
         |    (
         |     select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"1" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(defendant) as bg_name
         |            ,start_date as date
         |            ,md5(cleanup(CONCAT_WS('',case_no,start_date))) as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_open_announcement
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"1" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,replace_char(plaintiff) as yg_name
         |            ,replace_char(defendant) as bg_name
         |            ,start_date as date
         |            ,md5(cleanup(CONCAT_WS('',case_no,start_date))) as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_open_announcement
         |      where length(case_no) > 0 and ds > '0'
         |    )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)


    //立案信息预处理
    sql(
      s"""
         |insert ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_judicial_case_relation_pre partition(ds='$t1_ds',tn='court_register')
         |select
         |  judicase_id
         |  ,flag
         |  ,title
         |  ,case_type
         |  ,case_reason
         |  ,case_no
         |  ,court_name
         |  ,case_stage
         |  ,yg_name
         |  ,bg_name
         |  ,date
         |  ,detail_id
         |  ,case_amt
         |from (
         |     select
         |     *,row_number() over(partition by detail_id order by date desc) num
         |     from (
         |           select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.inc_ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |      union all
         |      select
         |            md5(cleanup(case_no)) as judicase_id
         |            ,"8" as flag
         |            ,title(plaintiff,defendant,case_reason) as title
         |            ,concat_ws('',case_type(case_no)) as case_type
         |            ,case_reason
         |            ,case_no
         |            ,court as court_name
         |            ,concat_ws('',case_stage(case_no)) as case_stage
         |            ,plaintiff as yg_name
         |            ,defendant as bg_name
         |            ,filing_date as date
         |            ,rowkey as detail_id
         |            ,0.0 as case_amt
         |      from $project.ads_company_court_register
         |      where length(case_no) > 0 and ds > '0'
         |     )
         |   )
         |where num = 1
         |""".stripMargin).show(10, false)

  }

  def calc(): Unit = {
    prepareFunctions(spark)
    map_2_json()
    case_no_trim_udf()
    registerCourtRank()
    spark.udf.register("name_aggs", new NameAggs(1000))
    spark.udf.register("case_reason", new CaseReasonAggs(1000))
    //预处理数据
    val cols = Seq("flag", "date", "detail_id", "name")
    val t1 = s"$project.inc_ads_company_court_announcement"
    val t2 = s"ads_judicial_case_relation_pre"
    var t2_ds = ds
    var t1_ds = ds
    if (StringUtils.isBlank(ds)) {
      t2_ds = BaseUtil.getPartion(t2, "wenshu", spark)
      //t1_ds = BaseUtil.getPartion(t1, spark)
      t1_ds = t2_ds
    }

    val t3 = "ads_judicial_case_relation_replace" //司法案件id交换表
    val t4 = "ads_judicial_case_incr_mapping"
    val t5 = s"base_company_mapping" //公司name和cid映射
    val t6 = s"ads_judicial_case_relation_replace_cids" //公司name和cid映射

    val t5_ds = BaseUtil.getPartion(t5, spark) //映射表分区

    //替换司法案件id
        sql(
          s"""
             |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.$t3 partition (ds = '$t1_ds')
             |SELECT
             |   judicase_id
             |   ,flag
             |   ,title
             |   ,case_type
             |   ,case_reason
             |   ,case_no
             |   ,court_name
             |   ,case_stage
             |   ,lable
             |   ,map_2_json(${getStrToMap(cols)}) as detail
             |   ,yg_name
             |   ,bg_name
             |   ,date
             |   ,detail_id
             |   ,case_amt
             |FROM
             |(
             |   SELECT  COALESCE(b.judicase_id,a.new_judicase_id) judicase_id
             |           ,a.flag
             |           ,a.title
             |           ,a.case_type
             |           ,a.case_reason
             |           ,case_no_trim(a.case_no) as case_no
             |           ,a.court_name
             |           ,a.case_stage
             |           ,case_label(a.flag) lable
             |           ,a.yg_name
             |           ,a.bg_name
             |           ,a.date
             |           ,a.detail_id
             |           ,a.case_amt
             |           ,a.bg_name as name
             |   FROM    (
             |     SELECT
             |        judicase_id
             |        ,flag
             |        ,title
             |        ,case_type
             |        ,case_reason
             |        ,case_no_trim(case_no) as case_no
             |        ,court_name
             |        ,case_stage
             |        ,replace_char(yg_name) as yg_name
             |        ,replace_char(bg_name) as bg_name
             |        ,date
             |        ,detail_id
             |        ,case_amt
             |        ,md5(CLEANUP(case_no_trim(case_no))) as new_judicase_id
             |     FROM $project.$t2
             |     WHERE ds= '$t2_ds' and tn not in ('wenshu','zxr','zxr_person','company_dishonest_info','company_dishonest_info_person')
             |           and case_no_trim(case_no) is not null
             |           and date is not null and length(date) = 19
             |   ) a
             |   LEFT JOIN (
             |     SELECT case_no_trim(case_no) as case_no,max(judicase_id) judicase_id
             |     FROM $project.$t2
             |     WHERE ds = '$t2_ds' and tn in ('wenshu','zxr','zxr_person','company_dishonest_info','company_dishonest_info_person')
             |     and case_no_trim(case_no) is not null
             |     GROUP BY case_no
             |   ) b
             |   ON  CLEANUP(a.case_no) = CLEANUP(b.case_no)
             |   UNION ALL
             |   SELECT   judicase_id
             |           ,flag
             |           ,title
             |           ,case_type
             |           ,case_reason
             |           ,case_no_trim(case_no) as case_no
             |           ,court_name
             |           ,case_stage
             |           ,case_label(flag) lable
             |           ,replace_char(yg_name) as yg_name
             |           ,replace_char(bg_name) as bg_name
             |           ,date
             |           ,detail_id
             |           ,case_amt
             |           ,replace_char(bg_name) as name
             |   FROM $project.$t2
             |   WHERE ds = '$t2_ds' and tn in ('wenshu','zxr','zxr_person','company_dishonest_info','company_dishonest_info_person')
             |         and case_no_trim(case_no) is not null
             |         and date is not null and length(date) = 19
             |)
             |""".stripMargin).show(10, false)

        //name 替换 cid
        sql(
          s"""
             |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.$t6 partition (ds = '$t1_ds')
             |SELECT
             |        a.judicase_id
             |        ,flag
             |        ,title
             |        ,case_type
             |        ,case_reason
             |        ,case_no
             |        ,court_name
             |        ,case_stage
             |        ,lable
             |        ,detail
             |        ,yg_name
             |        ,bg_name
             |        ,DATE
             |        ,detail_id
             |        ,case_amt
             |        ,coalesce(b.cids,'') AS cids
             |FROM    (
             |        SELECT  *
             |        FROM    $project.$t3
             |        WHERE   ds = '$t1_ds'
             |        ) a
             |LEFT JOIN (
             |        SELECT
             |                judicase_id
             |                ,sort(concat_ws(',',collect_set(cid)),',') cids
             |        FROM    (
             |                SELECT
             |                        e.judicase_id
             |                        ,f.new_cid cid
             |                FROM    (
             |                        SELECT  *
             |                        FROM    (
             |                                SELECT
             |                                        yg_name AS names
             |                                        ,judicase_id
             |                                FROM    $project.$t3
             |                                WHERE   ds = '$t1_ds' AND length(cleanup(yg_name)) >4
             |                                UNION ALL
             |                                SELECT
             |                                        bg_name AS names
             |                                        ,judicase_id
             |                                FROM    $project.$t3
             |                                WHERE   ds = '$t1_ds' AND length(cleanup(bg_name)) >4
             |                                ) a
             |                        LATERAL VIEW explode(split(names,',')) t AS name
             |                        ) e
             |                JOIN (
             |                        SELECT
             |                                cname
             |                                ,concat_ws(',',collect_set(new_cid)) AS new_cid
             |                        FROM    $project.$t5
             |                        WHERE   ds = '$t5_ds' AND length(cleanup(cname)) >4 AND company_type not in ('2','8')
             |                        GROUP BY cname
             |                     ) f
             |                ON      cleanup(e.name) = cleanup(f.cname)
             |                )
             |        GROUP BY judicase_id
             |          ) b
             |ON      a.judicase_id = b.judicase_id
             |""".stripMargin)

    val second_ds = getSecondLastPartitionOrElse(t6, "0")
    println(s"calc ds: $t2_ds, par ds : $t1_ds, second_ds : $second_ds")

    //找出增量数据
    //    sql(
    //      s"""
    //         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $project.$t4
    //         |SELECT  coalesce(a.judicase_id,b.judicase_id)judicase_id
    //         |        ,CASE WHEN a.judicase_id IS NULL THEN 1 ELSE 0 END
    //         |FROM    (
    //         |            SELECT  judicase_id
    //         |                    ,md5(concat_ws('', sort(concat_ws(',',collect_set(concat_ws('',flag,detail_id))),','), sort(concat_ws(',',collect_set(cids)),','))) r1
    //         |            FROM    $project.$t6
    //         |            WHERE   ds = '$t1_ds'
    //         |            GROUP BY judicase_id
    //         |        ) a
    //         |FULL JOIN (
    //         |              SELECT  judicase_id
    //         |                      ,md5(concat_ws('', sort(concat_ws(',',collect_set(concat_ws('',flag,detail_id))),','), sort(concat_ws(',',collect_set(cids)),','))) r2
    //         |              FROM    $project.$t6
    //         |              WHERE   ds = '$second_ds'
    //         |              GROUP BY judicase_id
    //         |          ) b
    //         |ON  r1 = r2
    //         |WHERE   r1 IS NULL OR r2 IS NULL
    //         |""".stripMargin)

    //司法案件主表
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci.ads_judicial_case_relation_r1
         |SELECT
         |    concat_ws('',x.judicase_id) judicase_id,
         |    title       ,
         |    case_type   ,
         |    case_reason ,
         |    case_no     ,
         |    court_name  ,
         |    case_stage  ,
         |    lable       ,
         |    detail      ,
         |    name_aggs['yg_name'] yg_name,
         |    name_aggs['bg_name'] bg_name,
         |    case_amt    ,
         |    date        ,
         |    court_level ,
         |    0 deleted   ,
         |    cids
         |FROM
         |(
         |SELECT  judicase_id
         |        ,max(title) title
         |        ,concat_ws(',',collect_set(case_type)) case_type
         |        ,case_reason(case_reason,date,flag) case_reason
         |        ,concat_ws(',',collect_set(case_no)) case_no
         |        ,concat_ws(',',collect_set(court_name)) court_name
         |        ,last_stage(concat_ws(' ',collect_set(case_stage))) case_stage
         |        ,trim_black(concat_ws(',',max(case_type),collect_set(lable))) lable
         |        ,null as detail
         |        ,max(case_amt) AS case_amt
         |        ,max(date) AS date
         |        ,trim_black(concat_ws(',',collect_set(court_level))) court_level
         |        ,concat_ws(',',collect_set(cids)) cids
         |        ,name_aggs(yg_name,bg_name,flag,date) name_aggs
         |FROM    (
         |        SELECT  a.*
         |        FROM    (
         |                   SELECT  judicase_id,flag,title,case_type,case_reason,case_no,court_name,case_stage,lable,yg_name,bg_name,date,case_amt,cids
         |                   ,court_level(court_name) court_level
         |                   FROM    $project.$t6
         |                   WHERE   ds = '$t1_ds'
         |                ) a
         |        )
         |GROUP BY judicase_id
         |)x
         |-- JOIN
         |-- (
         |--    select *
         |--    from $project.$t4
         |-- ) y on x.judicase_id = y.judicase_id
         |""".stripMargin).show(20, false)

    //明细表
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE winhc_eci.ads_judicial_case_relation_r2
         |SELECT
         |    id,
         |    concat_ws('',x.judicase_id) judicase_id,
         |    title       ,
         |    case_type   ,
         |    case_reason ,
         |    case_no     ,
         |    court_name  ,
         |    case_stage  ,
         |    lable       ,
         |    detail      ,
         |    name_aggs['yg_name'] yg_name,
         |    name_aggs['bg_name'] bg_name,
         |    last_date   ,
         |    0 deleted
         |FROM
         |(
         |SELECT  md5(concat_ws('',concat_ws('',judicase_id),CLEANUP(case_no))) id
         |        ,judicase_id
         |        ,max(first_title) title
         |        ,case_type(max(case_no)) as case_type
         |        ,case_reason(case_reason,date,flag) case_reason
         |        ,case_no
         |        ,max(court_name) court_name
         |        ,case_stage(max(case_no)) as case_stage
         |        ,trim_black(concat_ws(',',max(case_type),collect_set(lable))) lable
         |        ,concat('[',concat_ws(',',collect_set(detail)),']') detail
         |        ,max(last_date) last_date
         |        ,name_aggs(yg_name,bg_name,flag,date) name_aggs
         |FROM    (
         |        SELECT  a.*
         |                ,first_value(title) OVER (PARTITION BY a.judicase_id ORDER BY date ASC ) AS first_title
         |                ,first_value(date) OVER (PARTITION BY a.judicase_id ORDER BY date DESC ) AS last_date
         |        FROM    (
         |                   SELECT  *
         |                   FROM    $project.$t6
         |                   WHERE   ds = '$t1_ds' AND length(lable) > 0
         |                )a
         |)
         |GROUP BY judicase_id
         |         ,case_no
         |) x
         |-- JOIN
         |-- (
         |--    select *
         |--    from $project.$t4
         |-- ) y on x.judicase_id = y.judicase_id
         |""".stripMargin).show(10, false)

    //建表
    createIndex()

  }

  def createIndex(i1: String = "judicial_case_v", i2: String = "judicial_case_detail_v"): Unit = {
    val bizdate = BaseUtil.getYesterday()
    val index1 = s"$i1$bizdate"
    val index2 = s"$i2$bizdate"
    val client = EsQueryUtils.getClient()
    //建表
    EsQueryUtils.createTable(index1, EsQueryUtils.judicial_case, client)
    EsQueryUtils.createTable(index2, EsQueryUtils.judicial_case_detail, client)
    client.close()
  }

  def updateIndex(pre: String = "winhc_", i1: String = "judicial_case_v", i2: String = "judicial_case_detail_v"): Unit = {
    val bizdate = BaseUtil.getYesterday()
    val aliases1 = s"${pre}${i1.replace("_v","")}"
    val aliases2 = s"${pre}${i2.replace("_v","")}"
    val index1 = s"$i1$bizdate"
    val index2 = s"$i2$bizdate"
    val client = EsQueryUtils.getClient()

    if(!EsQueryUtils.tableExists(index1,client) || !EsQueryUtils.tableExists(index2,client)){
      println(s"index not exists $index1 -- $index2 ...")
      sys.exit(-1)
    }
    //获取index
    val list1 = EsQueryUtils.getAliases(aliases1, client)
    val list2 = EsQueryUtils.getAliases(aliases2, client)
    //移除索引
    list1.foreach(updateAliases(_, aliases1, "remove", client))
    list2.foreach(updateAliases(_, aliases2, "remove", client))
    //新增索引
    updateAliases(index1, aliases1, "add", client)
    updateAliases(index2, aliases2, "add", client)
    val list3 = EsQueryUtils.getIndexs(index1.substring(0, index1.indexOf("2")), client)
    val list4 = EsQueryUtils.getIndexs(index2.substring(0, index1.indexOf("2")), client)
    //删除多余索引
    if (list3.size > 2 && list4.size > 2) {
      EsQueryUtils.deleteIndex(list3.head, client)
      EsQueryUtils.deleteIndex(list4.head, client)
    }
    client.close()
  }

}