package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.udf.CompanyMapping
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 增量cids转换
 */

case class CompanyIncrForCidsUtils(s: SparkSession,
                                   project: String, //表所在工程名
                                   mainTableName: String, //主表名（不加前辍）
                                   sublistTableName: String, //子表（不加前辍）
                                   dupliCols: Seq[String] // 去重列
                                  ) extends LoggingUtils with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  val tabMapping =
    Map("company_court_open_announcement" -> ("litigant_cids", ";") //开庭公告
      , "company_send_announcement" -> ("litigant_cids", ",") //送达公告
    )

  val funMap =
    Map("company_court_announcement.case_no" -> "case_no(content)", //法院公告
      "company_court_announcement.plaintiff" -> "replace_char(plaintiff)",
      "company_court_announcement.litigant" -> "replace_char(litigant)"
    )

  val rowkey_mapping: Map[String, String] =
    Map(
      "company_patent.pub_number" -> s"rowkey_trans(pub_number,'$mainTableName')",//专利
      "company_patent.app_number" -> s"rowkey_trans(app_number,'$mainTableName')"
    )

  //转换字段
  def trans(s: String): String = {
    val key = mainTableName + "." + s
    var res = s
    if (funMap.contains(key)) {
      res = funMap(key)
    }
    res
  }

  def trans2(s: String) = {
    val key = mainTableName + "." + s
    var res = s
    if (rowkey_mapping.contains(key)) {
      res = rowkey_mapping(key)
    }
    res
  }

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    prepareFunctions(spark)
    val (split_cols, delimiter) = tabMapping.getOrElse(mainTableName, ("cids", ";"))

    val inc_ods_company = s"${project}.inc_ods_company" //每日公司基本信息增量
    val ads_company_tb = s"${project}.ads_$mainTableName" //存量ads主表数据
    val ads_company_tb_list = s"${project}.ads_$sublistTableName" //存量子表数据 用于读取表字段
    val inc_ods_company_tb = s"${project}.inc_ods_$mainTableName" //增量数据ods 主表
    val inc_ads_company_tb = s"${project}.inc_ads_$mainTableName" //增量数据ads 主表
    val inc_ads_company_tb_list = s"${project}.inc_ads_$sublistTableName" //增量数据ads 子表

    val sublistTableFieldName = spark.table(ads_company_tb_list).columns.filter(s => {
      !s.equals("ds") && !s.equals("new_cid") && !s.equals("rowkey") && !s.equals("cids") && !s.equals("new_cids") && !s.equals("cid")
    }).seq

    val sublistRes = spark.table(inc_ads_company_tb_list).columns.filter(s => {
      !s.equals("ds")
    }).map(trans).seq


    val subRes = spark.table(inc_ads_company_tb).columns.filter(s => {
      !s.equals("ds")
    }).map(trans).seq

    //存量表ads最新分区
    val remainDs = BaseUtil.getPartion(ads_company_tb, spark)

    //增量ads最后一个分区
    val lastDsIncAds = BaseUtil.getPartion(inc_ads_company_tb, spark)

    val list = sql(s"show partitions $inc_ods_company_tb").collect.toList.map(_.getString(0).split("=")(1))
    //增量ods第一个分区
    val firstDsIncOds = list.head
    //增量ods最后一个分区//落表分区
    val lastDsIncOds = list.last
    //执行分区
    var runDs = ""
    //第一次run
    if (StringUtils.isBlank(lastDsIncAds)) {
      runDs = firstDsIncOds
    } else { //非第一次分区时间 + 1天
      runDs = BaseUtil.atDaysAfter(1, lastDsIncAds)
    }

    val cols_md5 = dupliCols.filter(!_.equals("new_cid")).map(trans).map(trans2)

    //增量ods和增量ads最后一个分区相等,跳出
    if (lastDsIncOds.equals(lastDsIncAds)) {
      println("inc_ods equals inc_ads ds ,please delete last ds !!!")
      //runDs = lastDsIncOds
      val l1 = sql(s"show partitions $inc_ads_company_tb").collect.toList.map(_.getString(0).split("=")(1)).sorted
      if (l1.size > 1) {
        runDs = BaseUtil.atDaysAfter(1, l1(l1.size - 2))
      } else {
        runDs = firstDsIncOds
      }
      //sys.exit(-1)
    }

    println(
      s"""
         |cols_md5:$cols_md5
         |remainDs:$remainDs
         |lastDsIncOds:$lastDsIncOds
         |lastDsIncAds:$lastDsIncAds
         |runDs:$runDs
         |firstDsIncOds:$firstDsIncOds
         |""".stripMargin)

    //table字段
    val columns: Seq[String] = spark.table(ads_company_tb).schema.map(_.name).filter(s => {
      !s.equals("ds") && !s.equals("cid") && !s.equals("new_cid") && !s.equals("rowkey") && !s.equals("cids") && !s.equals("new_cids") && !s.equals("new_litigant_cids") && !s.equals("litigant_cids")
    })

    //判断字段是否有重复字段
    var f = "flag"
    if (sublistTableFieldName.contains(f)) {
      f = "update_flag"
    }

    //mapping 映射关系
    sql(
      s"""
         |SELECT  cid,current_cid as new_cid
         |FROM    ${inc_ods_company}
         |WHERE   ds >= ${runDs}
         |AND     cid IS NOT NULL
         |AND     current_cid IS NOT NULL
         |GROUP BY cid,current_cid
         |""".stripMargin).createOrReplaceTempView("mapping")

    //增量打平
    sql(
      s"""
         |SELECT  *
         |FROM    ${inc_ods_company_tb} a
         |LATERAL VIEW explode(split($split_cols,'$delimiter')) b AS cid
         |WHERE   ds >= ${runDs}
         |AND     $split_cols IS NOT NULL
         |AND     trim($split_cols) <> ''
         |""".stripMargin).createOrReplaceTempView("incr_tb")


    //替换cid,去重,复制老数据
    sql(
      s"""
         |INSERT OVERWRITE TABLE $inc_ads_company_tb_list PARTITION(ds='$lastDsIncOds')
         |SELECT
         |        ${sublistRes.mkString(",")}
         |FROM    (
         |            SELECT  CONCAT_WS( '_',new_cid,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")})))) AS rowkey
         |                    ,$f
         |                    ,new_cid
         |                    ,cid
         |                    ,${sublistTableFieldName.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',${dupliCols.map(trans).map(trans2)mkString(",")})) ORDER BY update_time DESC ) num
         |            FROM    (
         |                        SELECT  "0" AS $f
         |                                ,CAST(new_cid AS STRING) AS new_cid
         |                                ,CAST(cid AS STRING) AS cid
         |                                ,${sublistTableFieldName.mkString(",")}
         |                        FROM    (
         |                                    SELECT  *
         |                                    FROM    (
         |                                                SELECT  c.*
         |                                                        ,coalesce(d.new_cid,c.cid) AS new_cid
         |                                                FROM    incr_tb c
         |                                                LEFT JOIN mapping d
         |                                                ON      c.cid = d.cid
         |                                            ) e
         |                                ) f
         |                        UNION ALL
         |                        SELECT  "1" AS $f
         |                                ,CAST(new_cid AS STRING) AS new_cid
         |                                ,CAST(cid AS STRING) AS cid
         |                                ,${sublistTableFieldName.mkString(",")}
         |                        FROM    (
         |                                    SELECT  a.new_cid
         |                                            ,a.cid
         |                                            ,${sublistTableFieldName.mkString(",")}
         |                                    FROM    mapping a
         |                                    JOIN    (
         |                                                SELECT  new_cid AS cid
         |                                                        ,${sublistTableFieldName.mkString(",")}
         |                                                FROM    ${inc_ads_company_tb_list}
         |                                                WHERE   ds > ${remainDs} AND  ds <  ${runDs}
         |                                                UNION ALL
         |                                                SELECT  new_cid AS cid
         |                                                        ,${sublistTableFieldName.mkString(",")}
         |                                                FROM    ${ads_company_tb_list}
         |                                                WHERE   ds >= ${remainDs}
         |                                            ) b
         |                                    ON      a.cid = b.cid
         |                                ) c
         |                    ) e
         |            WHERE   cleanup(CONCAT_WS('',${cols_md5.mkString(",")})) IS NOT NULL
         |            AND     trim(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))) <> ''
         |        ) x
         |WHERE   num = 1
         |""".stripMargin)

    //主表按照id去重落库
    sql(
      s"""
         |INSERT OVERWRITE TABLE  $inc_ads_company_tb PARTITION(ds='$lastDsIncOds')
         |SELECT  ${subRes.mkString(",")}
         |FROM    (
         |            SELECT  $split_cols,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY cleanup(CONCAT_WS('',${cols_md5.mkString(",")})) ORDER BY update_time DESC ) num
         |                    ,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))) AS rowkey
         |            FROM    ${inc_ods_company_tb}
         |            WHERE   ds >= ${runDs}
         |            AND     $split_cols IS NOT NULL
         |            AND     trim($split_cols) <> ''
         |        ) a
         |WHERE   num = 1
         |""".stripMargin)

    //同步增量索引表数据
    val colsList = sublistTableFieldName ++ Seq("new_cid")
    MaxComputer2Phoenix(
      spark,
      colsList,
      inc_ads_company_tb_list,
      sublistTableName,
      lastDsIncOds,
      s"CONCAT_WS('_',new_cid,md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")}))))"
    ).syn()

    //同步增量主表数据
    val cols = columns ++ Seq(s"$split_cols")
    MaxComputer2Phoenix(
      spark,
      cols,
      inc_ads_company_tb,
      mainTableName,
      lastDsIncOds,
      s"md5(cleanup(CONCAT_WS('',${cols_md5.mkString(",")})))"
    ).syn()

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}