package com.winhc.bigdata.spark.utils

import java.util.Date

import com.winhc.bigdata.spark.udf.{BaseFunc, CompanyMapping}
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @Author yyn
 * @Date 2020/10/14
 * @Description 企业CID补全
 */
case class Company_Completion_Utils(s: SparkSession,
                                    project: String, //表所在工程名
                                    isNeedVerify: Boolean //是否需要验证，如所有表增量表的分区是否对齐（相等）
                                  ) extends LoggingUtils with BaseFunc with CompanyMapping {
  @(transient@getter) val spark: SparkSession = s

  def calc(): Unit = {
    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    prepareFunctions(spark)

    //参与补全的表
    var mapTables = new mutable.HashMap[String, (String, String, String, String)]()
    mapTables("company_zxr") = ("cids", "cname", "card", "1")
    mapTables("company_dishonest_info") = ("cid", "name", "card_num", "2")
    mapTables("company_zxr_final_case") = ("cid", "name", "identity_num", "3")
    mapTables("company_zxr_restrict") = ("cid", "coalesce(company_name,company_info)", "identity_num", "4")
    var lastDsIncOds: String = ""
    var minDs: String = ""
    var maxDs: String = ""
    mapTables.foreach(m => {
      if (lastDsIncOds == "") {
        lastDsIncOds = BaseUtil.getPartion("inc_ods_" + m._1, spark)
        minDs = lastDsIncOds
        maxDs = minDs
      }
      else {
        if (Integer.parseInt(lastDsIncOds) < Integer.parseInt(BaseUtil.getPartion("inc_ods_" + m._1, spark))) {
          minDs = lastDsIncOds
          maxDs = BaseUtil.getPartion("inc_ods_" + m._1, spark)
        }
        else {
          minDs = BaseUtil.getPartion("inc_ods_" + m._1, spark)
          maxDs = lastDsIncOds
        }
      }
    })
    //增量数据表没有准备好（无分区）时错误退出
    if (StringUtils.isBlank(minDs)) {
      println("all tables have not inc's partition !!!")
      sys.exit(-1)
    }
    //要求验证时发现各个表的分区未对齐（不相等）时错误退出
    if (isNeedVerify == true && minDs != maxDs) {
      println("not all tables have the same partition of newest !!!")
      sys.exit(-1)
    }
    id_card_trim_udf()
    lastDsIncOds = minDs
    spark.sparkContext.setJobDescription(s"补全企业cid:${mapTables.size}个表+企业映射表聚合($lastDsIncOds)")
    sql(mapTables.map(m => {
      s"""
         |SELECT SPLIT(${m._2._1},';')[0] AS cid, ${m._2._2} AS company_name, ${m._2._3} AS organization_code, ${m._2._4} AS source, ${m._2._4} AS flag
         |FROM $project.ods_${m._1}
         |WHERE ds>'0' AND ${m._2._2} IS NOT NULL AND LENGTH(${m._2._2})>=5
         |UNION ALL
         |SELECT SPLIT(${m._2._1},';')[0] AS cid, ${m._2._2} AS company_name, ${m._2._3} AS organization_code, ${m._2._4} AS source, ${m._2._4} AS flag
         |FROM $project.inc_ods_${m._1}
         |WHERE ds>'0' AND ${m._2._2} IS NOT NULL AND LENGTH(${m._2._2})>=5
         |""".stripMargin
    }).toArray.mkString(" UNION ALL ")
    ).createOrReplaceTempView("tmp_company_cloze_1")

    //1、根据姓名和案号补全身份证号码，未去重是为了后续根据姓名和公司来补全
    sql(
      s"""
         |SELECT cid, company_name, organization_code, source, flag
         |FROM(
         |  SELECT cid, company_name, organization_code, source, flag
         |        ,ROW_NUMBER() OVER (PARTITION BY company_name ORDER BY cid DESC) num
         |  FROM tmp_company_cloze_1
         |)
         |WHERE num=1
         |""".stripMargin
    ).createOrReplaceTempView("tmp_company_cloze_2")
    //2、根据姓名和公司来补全身份证号码
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} table $project.ads_company_cloze partition(ds='$lastDsIncOds')
         |SELECT cid, company_name, organization_code, source, flag
         |FROM(
         |  SELECT IF(A.cid IS NULL AND B.new_cid IS NOT NULL,B.new_cid,A.cid) AS cid
         |        ,A.company_name
         |        ,A.organization_code
         |        ,A.source
         |        ,IF(A.cid IS NULL AND B.new_cid IS NOT NULL,-1,A.flag) AS flag
         |        ,ROW_NUMBER() OVER (PARTITION BY A.cid,A.company_name ORDER BY A.cid DESC) num
         |  FROM tmp_company_cloze_2 A
         |  LEFT JOIN $project.base_company_mapping B
         |  ON A.company_name=B.cname
         |)
         |WHERE num=1
         |""".stripMargin
    )
    //    CompanyIncSummary(spark, project, tableName, "new_cid", dupliCols).calc

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}
object Company_Completion_Utils {

  def main(args: Array[String]): Unit = {

    val Array(project, flag) = args
    println(
      s"""
         |project: $project
         |flag: $flag
         |""".stripMargin)
    if (args.length < 2) {
      println("请输入 project:项目, tableName:表名, flag:标识!!!")
      sys.exit(-1)
    }
    val config = mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_eci_dev",
      "spark.hadoop.odps.spark.local.partition.amt" -> "100"
    )
    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)
    Company_Completion_Utils(spark, project, flag.toBoolean).calc()
    spark.stop()
  }
}



