package com.winhc.bigdata.spark.utils

import org.apache.commons.lang3.StringUtils

/**
 * @Author: XuJiakai
 * @Date: 2020/8/31 10:42
 * @Description:
 */
object case_connect_utils {

  def isConnect(current_case_party_list_org: Seq[String], connect_case_party_list_org: Seq[String], current_case_no: String = "", connect_case_no: String = "", current_court_name: String = "", connect_court_name: String = ""): Boolean = {
    val current_case_party_list = current_case_party_list_org.filter(StringUtils.isNotBlank)
    val connect_case_party_list = connect_case_party_list_org.filter(StringUtils.isNotBlank)

    if (current_case_party_list.isEmpty || connect_case_party_list.isEmpty) {
      return false
    }

    is_vague_word(current_case_party_list = current_case_party_list, connect_case_party_list = connect_case_party_list) match {
      case 1 => {
        vague_match(position = 0, current_case_party_list
          , connect_case_party_list, current_case_no, connect_case_no, current_court_name, connect_court_name
        )
      }
      case 2 => {
        vague_match(position = 1, current_case_party_list
          , connect_case_party_list, current_case_no, connect_case_no, current_court_name, connect_court_name
        )
      }
      case _ => {
        precise_match(current_case_party_list, connect_case_party_list)
      }
    }
  }

  private def vague_match(position: Int = 1,
                          current_case_party_list: Seq[String]
                          , connect_case_party_list: Seq[String]
                          , current_case_no: String, connect_case_no: String
                          , current_court_name: String, connect_court_name: String
                         ): Boolean = {
    var current_party: Seq[String] = null
    var connect_party: Seq[String] = null
    position match {
      case 1 => {
        current_party = connect_case_party_list
        connect_party = current_case_party_list
      }
      case _ => {
        current_party = current_case_party_list
        connect_party = connect_case_party_list
      }
    }

    for (char <- current_party.mkString("").toCharArray.map(_.toString).filter(vague_word.contains(_))) {
      for (userName <- current_party) {
        for (splitUser <- userName.replace(char,"\001").split("\001")) {
          val all_str = connect_party.mkString("")
          if (StringUtils.isNotEmpty(splitUser) && (!all_str.contains(splitUser.substring(0, 1)))) {
            return false
          }
        }
      }
    }
    case_no_match(current_case_no = current_case_no, connect_case_no = connect_case_no) || court_name_match(connect_court_name = connect_court_name, current_court_name = current_court_name)
  }

  private def case_no_match(current_case_no: String, connect_case_no: String): Boolean = current_case_no.equals(connect_case_no)

  private def court_name_match(current_court_name: String, connect_court_name: String): Boolean = current_court_name.equals(connect_court_name)

  private def precise_match(current_case_party_list: Seq[String]
                            , connect_case_party_list: Seq[String]
                           ): Boolean = {
    var num = 0
    for (userName <- connect_case_party_list) {
      if (current_case_party_list.contains(userName))
        num = num + 1
    }
    num >= connect_case_party_list.size / 2
  }

  val vague_word: Seq[String] = Seq("某", "*", "＊", "ｘ", "ⅹ", "x", "X", "×")

  private def is_vague_word(current_case_party_list: Seq[String], connect_case_party_list: Seq[String]): Int = {
    val current_vague = current_case_party_list.exists(r => vague_word.exists(r.equals(_)))
    if (current_vague) {
      return 1
    }
    val connect_vague = connect_case_party_list.exists(r => vague_word.exists(r.contains(_)))
    if (connect_vague) {
      return 2
    }
    0
  }

  def main(args: Array[String]): Unit = {
    val current_case_party_list: Seq[String] = Seq("乐视控股（北京）有限公司", "贾跃亭", "甘薇", "贾跃民", "武汉信用小额贷款股份有限公司")
    val connect_case_party_list: Seq[String] = Seq("贾跃民")

    val current_case_no = "(2017)鄂01民初3720号"
    val connect_case_no = "(2017)鄂01民初3720号"
    val current_court_name = "湖北省武汉市中级人民法院"
    val connect_court_name = "湖北省武汉市中级人民法院"
    //
    println(isConnect(current_case_party_list, connect_case_party_list, current_case_no, connect_case_no, current_court_name, connect_court_name))
    var userName = "jasdfja*sldfjk"
    var char = "*"

    println(userName.replace(char,"\001").split("\001").mkString("\t"))
  }
}
